/*------------------------------------------------------------------------*\
|                                                                          |
|           LISTINF.CMD  - Version 1.0 - Version Date 1995-04-07           |
|                 Copyright (C) 1995 by C F S Nevada, Inc.                 |
|                                                                          |
|                  by Dick Goran  - Voice    702-732-9616                  |
|                                 - FAX      702-732-3847                  |
|                                 - CIS      71154,2002                    |
|                                 - Internet dgoran@cfsrexx.com            |
|                                                                          |
| ------------------------------------------------------------------------ |
|  Requires: REXXLIB.DLL  - OS/2 REXX external function library            |
|                           (c) Copyright 1992-95 Quercus Systems          |
\*------------------------------------------------------------------------*/
/*

   Purpose:  This program will search all available and ready non-CD-ROM
             drives on the system and build a folder on the Desktop
             (INF File Collection) that contains the ability to view any
             of these .INF files from this central location.

   Function: The INF File Collection folder contains WPProgram
             objects for each of the .INF files. This approach was
             chosen, as opposed to simply creating shadows of the
             .INF files, so that the title line for each object
             could be constructed from both the title contained
             within the document in the .INF file as well as the
             full file system name of the .INF file. The file system
             name is converted to lowercase for aesthetics.

   Files:    This program will create LISTINF.DAT, and possibly LISTINF.LOG
             in the same directory where this program resides. the .DAT
             file is an internal representation of the compound array
             which holds the tree information and the .LOG file is
             created if assumed duplicate .INF files are found. The files
             included in the .LOG file MUST be examined manually to insure
             their validity.

   Warning:  If you have a large number of .INF files ion you system,
             your OS2.INI file may expand substantially as a result of
             running this program. The additional .INI file data is removed
             if the INF File Collection folder and its contents are deleted.

*/                                                                /* 0044 */
                                                                  /* 0045 */
   SIGNAL ON ERROR                  /* trap object time errors     */
   SIGNAL ON FAILURE                /* trap object time errors     */
   SIGNAL ON HALT                   /* trap object time errors     */
   SIGNAL ON NOVALUE                /* trap object time errors     */
   SIGNAL ON SYNTAX                 /* trap object time errors     */
                                                                  /* 0051 */
GBL. = ''             /* initialize stem */                       /* 0052 */
parse Arg             GBL.command_line                            /* 0053 */
parse Version         GBL.REXX_version .                          /* 0054 */
parse Source          GBL.operating_system,                       /* 0055 */
                      GBL.calling_environment,                    /* 0056 */
                      GBL.program_path_and_name                   /* 0057 */
GBL.environment     = 'OS2ENVIRONMENT'                            /* 0058 */
GBL.boot_drive      = LEFT( VALUE( 'RUNWORKPLACE',, GBL.environment ), 2 )
GBL.program_version = 1.0           /* version / mod of this program */
GBL.program_name    = FILESPEC( 'N', GBL.program_path_and_name )  /* 0061 */
GBL.program_path    = FILESPEC( 'D', GBL.program_path_and_name ) ||,
                      FILESPEC( 'P', GBL.program_path_and_name )  /* 0063 */
                                                                  /* 0064 */
parse var GBL.program_name,                                       /* 0065 */
   GBL.program_fn '.',                                            /* 0066 */
   GBL.program_fe                                                 /* 0067 */
call TIME 'E'                       /* set elapsed timer - sssss.uuuuu */
say 'Begin' TRANSLATE( GBL.program_name ) 'at' TIME('N')          /* 0069 */
call REGISTER_REQUIRED_FUNCTIONS                                  /* 0070 */
                                                                  /* 0071 */
GBl.log_file =,                                                   /* 0072 */
   GBL.program_path            ||,                                /* 0073 */
   TRANSLATE( GBL.program_fn ) || '.LOG'                          /* 0074 */
                                                                  /* 0075 */
GBL.log_file_line_count = 0                                       /* 0076 */
GBL.sort_max_length     = 0                                       /* 0077 */
GBL.title_max_length    = 0                                       /* 0078 */
GBL.id                  = 'C F S Inf Collection:'                 /* 0079 */
                                                                  /* 0080 */
/*------------------------*\                                      /* 0081 */
|  Box drawing characters  |                                      /* 0082 */
\*------------------------*/                                      /* 0083 */
GBL.box_draw = 'dbl. sgl. sgv. sgh.'/* global expose list                  */
GBL.width = 76                      /* log file output width               */
                                                                  /* 0086 */
dbl.h     = ''                     /* double line - horizontal            */
dbl.v     = ''                     /* double line - vertical              */
dbl.bl    = ''                     /* double line - bottom left           */
dbl.br    = ''                     /* double line - bottom right          */
dbl.tl    = ''                     /* double line - top left              */
dbl.tr    = ''                     /* double line - top right             */
dbl.t_d   = ''                     /* double line - "T" down              */
dbl.t_l   = ''                     /* double line - "T" left              */
dbl.t_r   = ''                     /* double line - "T" right             */
dbl.t_u   = ''                     /* double line - "T" up                */
dbl.cross = ''                     /* double line - cross bar             */
                                                                  /* 0098 */
sgl.h     = ''                     /* single line - horizontal            */
sgl.v     = ''                     /* single line - vertical              */
sgl.bl    = ''                     /* single line - bottom left           */
sgl.br    = ''                     /* single line - bottom right          */
sgl.tl    = ''                     /* single line - top left              */
sgl.tr    = ''                     /* single line - top right             */
sgl.t_d   = ''                     /* single line - "T" down              */
sgl.t_l   = ''                     /* single line - "T" left              */
sgl.t_r   = ''                     /* single line - "T" right             */
sgl.t_u   = ''                     /* single line - "T" up                */
sgl.cross = ''                     /* single line - cross bar             */
                                                                  /* 0110 */
sgh.h     = ''                     /* sgl horiz, dbl vert. - horizontal   */
sgh.v     = ''                     /* sgl horiz, dbl vert. - vertical     */
sgh.bl    = ''                     /* sgl horiz, dbl vert. - bottom left  */
sgh.br    = ''                     /* sgl horiz, dbl vert. - bottom right */
sgh.tl    = ''                     /* sgl horiz, dbl vert. - top left     */
sgh.tr    = ''                     /* sgl horiz, dbl vert. - top right    */
sgh.t_d   = ''                     /* sgl horiz, dbl vert. - "T" down     */
sgh.t_l   = ''                     /* sgl horiz, dbl vert. - "T" left     */
sgh.t_r   = ''                     /* sgl horiz, dbl vert. - "T" right    */
sgh.t_u   = ''                     /* sgl horiz, dbl vert. - "T" up       */
sgh.cross = ''                     /* sgl horiz, dbl vert. - cross bar    */
                                                                  /* 0122 */
sgv.h     = ''                     /* sgl vert, dbl horiz. - horizontal   */
sgv.v     = ''                     /* sgl vert, dbl horiz. - vertical     */
sgv.bl    = ''                     /* sgl vert, dbl horiz. - bottom left  */
sgv.br    = ''                     /* sgl vert, dbl horiz. - bottom right */
sgv.tl    = ''                     /* sgl vert, dbl horiz. - top left     */
sgv.tr    = ''                     /* sgl vert, dbl horiz. - top right    */
sgv.t_d   = ''                     /* sgl vert, dbl horiz. - "T" down     */
sgv.t_l   = ''                     /* sgl vert, dbl horiz. - "T" left     */
sgv.t_r   = ''                     /* sgl vert, dbl horiz. - "T" right    */
sgv.t_u   = ''                     /* sgl vert, dbl horiz. - "T" up       */
sgv.cross = ''                     /* sgl vert, dbl horiz. - cross bar    */
                                                                  /* 0134 */
                                                                  /* 0135 */
GBL.bksp               = '08'x                                    /* 0136 */
GBL.progress_list      = '\|/'                                   /* 0137 */
GBL.progress_subscript = 1                                        /* 0138 */
                                                                  /* 0139 */
/*-------------------------------*\                               /* 0140 */
|  Retrieve NLS dependent values  |                               /* 0141 */
\*-------------------------------*/                               /* 0142 */
GBL.sDate_character =,                                            /* 0143 */
   STRIP( SysIni( 'USER', 'PM_National', 'sDate' ), 'T', '00'x )  /* 0144 */
GBL.sDate_character = '/'           /* always / for SysFileTree */
                                                                  /* 0146 */
GBL.iDate_character =,                                            /* 0147 */
   STRIP( SysIni( 'USER', 'PM_National', 'iDate' ), 'T', '00'x )  /* 0148 */
                                                                  /* 0149 */
/*--------------------------------------------------------*\      /* 0150 */
|  Kill tree file if not command line parameter of k[eep]  |      /* 0151 */
\*--------------------------------------------------------*/      /* 0152 */
GBl.tree_file =,                                                  /* 0153 */
   GBL.program_path            ||,                                /* 0154 */
   TRANSLATE( GBL.program_fn ) || '.DAT'                          /* 0155 */
if ABBREV( TRANSLATE( GBL.command_line ), '/K' ) = 0 then         /* 0156 */
   do                                                             /* 0157 */
      call SysFileDelete GBL.tree_file                            /* 0158 */
   end                                                            /* 0159 */
                                                                  /* 0160 */
if STREAM( GBL.tree_file, 'C', 'QUERY EXISTS' ) = '' then        /* 0161 */
   do                                                             /* 0162 */
      /* assign values to stem from file */                       /* 0163 */
      call CVREAD GBL.tree_file, 'GBL.TREE_STEM'                  /* 0164 */
   end                                                            /* 0165 */
else                                                              /* 0166 */
   do                                                             /* 0167 */
      GBL.TREE_STEM.0 = 0                                         /* 0168 */
      call BUILD_TREE_STEM                                        /* 0169 */
   end                                                            /* 0170 */
call ARRAYSORT 'GBL.TREE_STEM', 1, GBL.TREE_STEM.0,,              /* 0171 */
               01, 80, 'D', 'C'                                   /* 0172 */
                                                                  /* 0173 */
/*------------------------------------------*\                    /* 0174 */
|  Build INF Files Folder Object on Desktop  |                    /* 0175 */
\*------------------------------------------*/                    /* 0176 */
folder_id        = '<' || GBL.id || ' Folder>'                    /* 0177 */
folder_location  = '<WP_DESKTOP>'                                 /* 0178 */
folder_class     = 'WPFolder'                                     /* 0179 */
folder_title     = 'INF File^Collection'                          /* 0180 */
folder_setup     =,                                               /* 0181 */
   'TITLE='    || folder_title || ';'          ||,                /* 0182 */
   'MINWIN=HIDE;'                              ||,                /* 0183 */
   'CCVIEW=NO;'                                ||,                /* 0184 */
   'ICONVIEW=FLOWED,NORMAL;'                   ||,                /* 0185 */
   'DETAILSVIEW=NORMAL;'                       ||,                /* 0186 */
   'ALWAYSSORT=NO;'                            ||,                /* 0187 */
   'OBJECTID=' || folder_id || ';'             ||,                /* 0188 */
   ''                                                             /* 0189 */
                                                                  /* 0190 */
call CHAROUT 'CON:', '   Begin building ' ||,                     /* 0191 */
                     folder_id || COPIES(' ', 3 )                 /* 0192 */
call SysCurState 'OFF'                                            /* 0193 */
                                                                  /* 0194 */
call SysCreateObject folder_class,,                               /* 0195 */
                     folder_title,,                               /* 0196 */
                     folder_location,,                            /* 0197 */
                     folder_setup,,                               /* 0198 */
                     'R'                                          /* 0199 */
if RESULT = 1 then                                               /* 0200 */
   do                                                             /* 0201 */
      say '   Unable to create ' ||,                              /* 0202 */
          folder_title           ||,                              /* 0203 */
          '. Should not occur.'                                   /* 0204 */
      call EOJ                                                    /* 0205 */
   end                                                            /* 0206 */
                                                                  /* 0207 */
/*---------------------------------*\                             /* 0208 */
|  Add INF files to the new folder  |                             /* 0209 */
\*---------------------------------*/                             /* 0210 */
lowercase = 'abcdefghijklmnopqrstuvwxyz'                          /* 0211 */
uppercase = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'                          /* 0212 */
                                                                  /* 0213 */
inf_class    = 'WPProgram'                                        /* 0214 */
inf_setup    =,                                                   /* 0215 */
   'CCVIEW=NO;'                    ||,                            /* 0216 */
   'DEFAULTVIEW=RUNNING;'          ||,                            /* 0217 */
   'EXENAME=?:\OS2\VIEW.EXE;'      ||,                            /* 0218 */
   'ICONRESOURCE=60 PMWP;'         ||,                            /* 0219 */
   'NOPRINT=YES;'                  ||,                            /* 0220 */
   'PROGTYPE=PM;'                  ||,                            /* 0221 */
   'PARAMETERS='                                                  /* 0222 */
                                                                  /* 0223 */
do ts = 1 to GBL.TREE_STEM.0                                      /* 0224 */
   if GBL.TREE_STEM.ts = '01'x then                               /* 0225 */
      do                                                          /* 0226 */
         leave ts                   /* we've reached the deleted dups */
      end                                                         /* 0228 */
                                                                  /* 0229 */
   call ADVANCE_PROGRESS_INDICATOR                                /* 0230 */
   parse value GBL.TREE_STEM.ts with,                             /* 0231 */
      inf_title 80,                                               /* 0232 */
      inf_date,                                                   /* 0233 */
      inf_time,                                                   /* 0234 */
      inf_size,                                                   /* 0235 */
      inf_crc,                                                    /* 0236 */
      inf_path_and_file_name                                      /* 0237 */
                                                                  /* 0238 */
   title_line_1 =,                                                /* 0239 */
      STRIP( inf_title )                                          /* 0240 */
   title_line_2 =,                                                /* 0241 */
      TRANSLATE( inf_path_and_file_name, lowercase, uppercase )   /* 0242 */
                                                                  /* 0243 */
   temp_setup = inf_setup                                              ||,
                inf_path_and_file_name ||                          ';' ||,
                'TITLE=' || title_line_1 || '^' || title_line_2 || ';' ||,
                'OBJECTID=' || '<' || GBL.id || inf_crc || '>;'   /* 0247 */
                                                                  /* 0248 */
   call SysCreateObject inf_class,,                               /* 0249 */
                        title_line_1 || '^' || title_line_2,,     /* 0250 */
                        folder_id,,                               /* 0251 */
                        temp_setup,,                              /* 0252 */
                        'F'                                       /* 0253 */
   if RESULT = 1 then                                            /* 0254 */
      do                                                          /* 0255 */
         call CHAROUT 'Con:', GBL.bksp || ' ' || '0D0A'x                ||,
                              '   Unable to create program object for ' ||,
                              inf_path_and_file_name                    ||,
                              '. Should not occur.  '             /* 0259 */
      end                                                         /* 0260 */
                                                                  /* 0261 */
end                                                               /* 0262 */
                                                                  /* 0263 */
call CHAROUT 'CON:', GBL.bksp || ' ' || '0D0A'x                   /* 0264 */
call SysCurState 'ON'                                             /* 0265 */
                                                                  /* 0266 */
/* call SysSetObjectData folder_id, 'ALWAYSSORT=YES;' */          /* 0267 */
                                                                  /* 0268 */
if GBL.log_file_line_count > 0 then                               /* 0269 */
   do                                                             /* 0270 */
      call STREAM GBL.log_file, 'C', 'CLOSE'                      /* 0271 */
      box_line.3 = dbl.v  || ' See: ' || GBL.log_file ||,         /* 0272 */
                   ' for a duplicate file list. '                    || dbl.v
      box_line.2 = dbl.v  || COPIES( ' ',   LENGTH(box_line.3) - 2 ) || dbl.v
      box_line.1 = dbl.tl || COPIES( dbl.h, LENGTH(box_line.3) - 2 ) || dbl.tr
      box_line.5 = dbl.bl || COPIES( dbl.h, LENGTH(box_line.3) - 2 ) || dbl.br
      box_line.4 = box_line.2                                     /* 0277 */
      box_line.0 = 5                                              /* 0278 */
      do b = 1 to box_line.0                                      /* 0279 */
         say COPIES( ' ', 3 ) || box_line.b                       /* 0280 */
      end                                                         /* 0281 */
   end                                                            /* 0282 */
                                                                  /* 0283 */
call EOJ 0                                                        /* 0284 */
                                                                  /* 0285 */
/*------------------------------------------------------------------------*\
|                                                                          |
|       Build stem of all .INF files indicating obsolete entries and       |
|                          identifying duplicates                          |
|                                                                          |
\*------------------------------------------------------------------------*/
BUILD_TREE_STEM:                                                  /* 0292 */
   Procedure expose,                                              /* 0293 */
      GBL. (GBL.box_draw)                                         /* 0294 */
                                                                  /* 0295 */
drive_list = SysDriveMap()                                        /* 0296 */
                                                                  /* 0297 */
do while drive_list = ''                                         /* 0298 */
   parse value drive_list with,                                   /* 0299 */
      drive_letter_colon,                                         /* 0300 */
      drive_list                                                  /* 0301 */
                                                                  /* 0302 */
   /*---------------------------------------------------*\        /* 0303 */
   |  Ignore CD-ROMs and hard drives that are not ready  |        /* 0304 */
   \*---------------------------------------------------*/        /* 0305 */
   if DOSDISK( 'U', drive_letter_colon ) \> 0 then iterate        /* 0306 */
   select                                                         /* 0307 */
      when DOSFILESYS( drive_letter_colon ) = 'FAT'  then nop     /* 0308 */
      when DOSFILESYS( drive_letter_colon ) = 'HPFS' then nop     /* 0309 */
      otherwise iterate                                           /* 0310 */
   end                                                            /* 0311 */
                                                                  /* 0312 */
   /*-----------------------------------------*\                  /* 0313 */
   |  Build & process stem of all directories  |                  /* 0314 */
   \*-----------------------------------------*/                  /* 0315 */
   call CHAROUT 'CON:',,                                          /* 0316 */
                '   Processing drive ' ||,                        /* 0317 */
                drive_letter_colon     ||,                        /* 0318 */
                ' looking for .INF files. '                       /* 0319 */
                                                                  /* 0320 */
   call SysFileTree drive_letter_colon || '\*.inf', 'temp_stem', 'S'
   if temp_stem.0 = 0 then                                        /* 0322 */
      do                                                          /* 0323 */
         call CHAROUT 'CON:', 'None found.' || '0D0A'x            /* 0324 */
         iterate                                                  /* 0325 */
      end                                                         /* 0326 */
                                                                  /* 0327 */
   call CHAROUT 'CON:', RIGHT( temp_stem.0, 4 ) || ' found.  '    /* 0328 */
   call SysCurState 'OFF'                                         /* 0329 */
   do temp = 1 to temp_stem.0                                     /* 0330 */
                                                                  /* 0331 */
      call ADVANCE_PROGRESS_INDICATOR                             /* 0332 */
      parse value temp_stem.temp with,                            /* 0333 */
         temp_date,                                               /* 0334 */
         temp_time,                                               /* 0335 */
         temp_size,                                               /* 0336 */
         temp_attr,                                               /* 0337 */
         temp_path_and_file_name                                  /* 0338 */
      temp_path_and_file_name = STRIP( temp_path_and_file_name )  /* 0339 */
                                                                  /* 0340 */
      select                                                      /* 0341 */
         when GBL.iDate_character = '0' then                      /* 0342 */
            do                                                    /* 0343 */
               parse value temp_date with,                        /* 0344 */
                  temp_mm,                                        /* 0345 */
                  (GBL.sDate_character),                          /* 0346 */
                  temp_dd,                                        /* 0347 */
                  (GBL.sDate_character),                          /* 0348 */
                  temp_yy                                         /* 0349 */
            end                                                   /* 0350 */
         when GBL.iDate_character = '1' then                      /* 0351 */
            do                                                    /* 0352 */
               parse value temp_date with,                        /* 0353 */
                  temp_dd,                                        /* 0354 */
                  (GBL.sDate_character),                          /* 0355 */
                  temp_mm,                                        /* 0356 */
                  (GBL.sDate_character),                          /* 0357 */
                  temp_yy                                         /* 0358 */
            end                                                   /* 0359 */
         when GBL.iDate_character = '2' then                      /* 0360 */
            do                                                    /* 0361 */
               parse value temp_date with,                        /* 0362 */
                  temp_yy,                                        /* 0363 */
                  (GBL.sDate_character),                          /* 0364 */
                  temp_mm,                                        /* 0365 */
                  (GBL.sDate_character),                          /* 0366 */
                  temp_dd                                         /* 0367 */
            end                                                   /* 0368 */
         otherwise                                                /* 0369 */
            do                                                    /* 0370 */
               say '   Unrecognized iDate character in OS2.INI file: ' ||
                   GBL.idate_character                            /* 0372 */
               call EOJ                                           /* 0373 */
            end                                                   /* 0374 */
      end                                                         /* 0375 */
      temp_yyyy = '19' || temp_yy                                 /* 0376 */
      if temp_yy < 80 then                                        /* 0377 */
         do                                                       /* 0378 */
            temp_yyyy = '20' || temp_yy                           /* 0379 */
         end                                                      /* 0380 */
                                                                  /* 0381 */
      /*--------------------------------------------------*\      /* 0382 */
      |  Prefix SysFileTree() data with title for sorting  |      /* 0383 */
      \*--------------------------------------------------*/      /* 0384 */
      temp_area = CHARIN( temp_path_and_file_name, 1, 188 )       /* 0385 */
      call STREAM temp_path_and_file_name, 'C', 'CLOSE'           /* 0386 */
      if LEFT( temp_area, 4 ) = 'HSP' || '01'x then              /* 0387 */
         do                                                       /* 0388 */
            /* ignore unrecognizeable format */                   /* 0389 */
            iterate temp                                          /* 0390 */
         end                                                      /* 0391 */
      parse value temp_area with,                                 /* 0392 */
         108,                                                     /* 0393 */
         temp_title,                                              /* 0394 */
         '00'x                                                    /* 0395 */
      if temp_title = '' then                                     /* 0396 */
         do                                                       /* 0397 */
            temp_title = TRANSLATE( FILESPEC( 'N', temp_path_and_file_name ) )
         end                                                      /* 0399 */
                                                                  /* 0400 */
      /*-----------------------------------------------*\         /* 0401 */
      |  Edit title removing any necessary punctuation  |         /* 0402 */
      \*-----------------------------------------------*/         /* 0403 */
      remove_characters = '<>*";'                                 /* 0404 */
      temp_title = TRANSLATE( temp_title,,                        /* 0405 */
                              COPIES( ' ', LENGTH(remove_characters) ),,
                              remove_characters )                 /* 0407 */
      temp_title = SPACE( temp_title )                            /* 0408 */
      GBL.title_max_length = MAX( GBL.title_max_length, LENGTH( temp_title ) )
                                                                  /* 0410 */
      /*---------------------*\                                   /* 0411 */
      |  Build combined stem  |                                   /* 0412 */
      \*---------------------*/                                   /* 0413 */
      ts = GBL.TREE_STEM.0 + 1                                    /* 0414 */
      GBL.TREE_STEM.0 = ts                                        /* 0415 */
      GBL.TREE_STEM.ts =,                                         /* 0416 */
         LEFT( temp_title, 80 )            ||,                    /* 0417 */
         temp_yyyy || RIGHT( temp_mm, 2, '0' ) || RIGHT( temp_dd, 2, '0' ),
         RIGHT( temp_time, 6, '0' ),                              /* 0419 */
         temp_size,                                               /* 0420 */
         FILECRC( temp_path_and_file_name ),                      /* 0421 */
         temp_path_and_file_name                                  /* 0422 */
      GBL.sort_max_length = MAX( GBL.sort_max_length, LENGTH( GBL.TREE_STEM.ts ) )
                                                                  /* 0424 */
   end temp                                                       /* 0425 */
                                                                  /* 0426 */
   call CHAROUT 'CON:', GBL.bksp || ' ' || '0D0A'x                /* 0427 */
   call SysCurState 'ON'                                          /* 0428 */
end                                                               /* 0429 */
                                                                  /* 0430 */
/*------------------------------------------*\                    /* 0431 */
|  Sort by file name and SysFileTree() data  |                    /* 0432 */
\*------------------------------------------*/                    /* 0433 */
call ARRAYSORT 'GBL.TREE_STEM', 1, GBL.TREE_STEM.0,,              /* 0434 */
               01, 80, 'A', 'C',,                                 /* 0435 */
               81, GBL.sort_max_length - 80, 'D', 'C'             /* 0436 */
                                                                  /* 0437 */
/*--------------------------------------*\                        /* 0438 */
|  Create log file entry for duplicates  |                        /* 0439 */
\*--------------------------------------*/                        /* 0440 */
call SysFileDelete GBL.log_file                                   /* 0441 */
prev. = ''                                                        /* 0442 */
call CHAROUT 'CON:', '   Processing .INF file list (' ||,         /* 0443 */
                     GBL.tree_stem.0                  ||,         /* 0444 */
                     ' entries used).'                            /* 0445 */
call SysCurState 'OFF'                                            /* 0446 */
                                                                  /* 0447 */
do ts = 1 to GBL.TREE_STEM.0                                      /* 0448 */
   call ADVANCE_PROGRESS_INDICATOR                                /* 0449 */
                                                                  /* 0450 */
   parse value GBL.TREE_STEM.ts with,                             /* 0451 */
      curr.title 80,                                              /* 0452 */
      curr.date,                                                  /* 0453 */
      curr.time,                                                  /* 0454 */
      curr.size,                                                  /* 0455 */
      curr.crc,                                                   /* 0456 */
      curr.path_and_file_name                                     /* 0457 */
   curr.title = STRIP( curr.title )                               /* 0458 */
                                                                  /* 0459 */
   if curr.title = prev.title then                                /* 0460 */
      do                                                          /* 0461 */
         /*-------------------------------*\                      /* 0462 */
         |  Add replaced file to LOG file  |                      /* 0463 */
         \*-------------------------------*/                      /* 0464 */
         GBL.TREE_STEM.ts = '01'x   /* kill duplicate entry */    /* 0465 */
         if curr.crc = prev.crc then                             /* 0466 */
            do                                                    /* 0467 */
               if GBL.log_file_line_count = 0 then                /* 0468 */
                  do                                              /* 0469 */
                     call CREATE_LOG_FILE_HEADER                  /* 0470 */
                  end                                             /* 0471 */
               call CREATE_LOG_FILE_DETAIL                        /* 0472 */
               call LINEOUT GBL.log_file, 'del ' || curr.path_and_file_name
               GBL.log_file_line_count = GBL.log_file_line_count + 1
            end                                                   /* 0475 */
         iterate ts                                               /* 0476 */
      end                                                         /* 0477 */
                                                                  /* 0478 */
   prev.title              = curr.title                           /* 0479 */
   prev.crc                = curr.crc                             /* 0480 */
   prev.date               = curr.date                            /* 0481 */
   prev.time               = curr.time                            /* 0482 */
   prev.size               = curr.size                            /* 0483 */
   prev.path_and_file_name = curr.path_and_file_name              /* 0484 */
                                                                  /* 0485 */
end                                                               /* 0486 */
                                                                  /* 0487 */
call CVWRITE GBL.tree_file, 'GBL.TREE_STEM'                       /* 0488 */
call CHAROUT 'CON:', GBL.bksp || ' ' || '0D0A'x                   /* 0489 */
call SysCurState 'ON'                                             /* 0490 */
                                                                  /* 0491 */
return                                                            /* 0492 */
                                                                  /* 0493 */
/*------------------------------------------------------------------------*\
|                                                                          |
|           Advance the twirling character to indicate progress            |
|                                                                          |
\*------------------------------------------------------------------------*/
ADVANCE_PROGRESS_INDICATOR:                                       /* 0499 */
   Procedure expose,                                              /* 0500 */
      GBL.                                                        /* 0501 */
                                                                  /* 0502 */
call CHAROUT 'CON:', GBL.bksp ||,                                 /* 0503 */
                    SUBSTR( GBL.progress_list, GBL.progress_subscript, 1 )
GBL.progress_subscript = GBL.progress_subscript + 1               /* 0505 */
if GBL.progress_subscript > LENGTH(GBL.progress_list) then        /* 0506 */
   GBL.progress_subscript = 1                                     /* 0507 */
return                                                            /* 0508 */
                                                                  /* 0509 */
                                                                  /* 0510 */
/*------------------------------------------------------------------------*\
|                                                                          |
|                Build & write log file detail line entries                |
|                                                                          |
\*------------------------------------------------------------------------*/
CREATE_LOG_FILE_DETAIL:                                           /* 0516 */
   Procedure expose,                                              /* 0517 */
      GBL. (GBL.box_draw),                                        /* 0518 */
      curr. prev.                                                 /* 0519 */
                                                                  /* 0520 */
parse value curr.date with,                                       /* 0521 */
   curr.year +4,                                                  /* 0522 */
   curr.mon  +2,                                                  /* 0523 */
   curr.day                                                       /* 0524 */
parse value prev.date with,                                       /* 0525 */
   prev.year +4,                                                  /* 0526 */
   prev.mon  +2,                                                  /* 0527 */
   prev.day                                                       /* 0528 */
                                                                  /* 0529 */
ln = 0                                                            /* 0530 */
ln = ln + 1; line.ln =,                                           /* 0531 */
   dbl.tl  || COPIES( dbl.h, GBL.width - 2 )      || dbl.tr       /* 0532 */
ln = ln + 1; line.ln =,                                           /* 0533 */
   dbl.v   || CENTER( curr.title, GBL.width - 2 ) || dbl.v        /* 0534 */
ln = ln + 1; line.ln =,                                           /* 0535 */
   sgh.t_r || COPIES( sgl.h, GBL.width - 2 )      || sgh.t_l      /* 0536 */
                                                                  /* 0537 */
ln = ln + 1; line.ln =,                                           /* 0538 */
   LEFT( dbl.v || ' ' ||,                                         /* 0539 */
         prev.path_and_file_name, GBL.width - 1 ) || dbl.v        /* 0540 */
ln = ln + 1; line.ln =,                                           /* 0541 */
   LEFT( dbl.v || COPIES( ' ', 3 ) ||,                            /* 0542 */
         prev.year'-'prev.mon'-'prev.day || COPIES(' ', 2 ) ||,   /* 0543 */
         RIGHT( prev.time, 6, '0' )      || COPIES(' ', 2 ) ||,   /* 0544 */
         RIGHT( EDIT(prev.size), 9 ),,                            /* 0545 */
            GBL.width - 1 ) || dbl.v                              /* 0546 */
                                                                  /* 0547 */
ln = ln + 1; line.ln =,                                           /* 0548 */
   dbl.v   || CENTER( '--- may replace ---', GBL.width - 2 ) || dbl.v
                                                                  /* 0550 */
ln = ln + 1; line.ln =,                                           /* 0551 */
   LEFT( dbl.v || ' ' ||,                                         /* 0552 */
         curr.path_and_file_name, GBL.width - 1 ) || dbl.v        /* 0553 */
ln = ln + 1; line.ln =,                                           /* 0554 */
   LEFT( dbl.v || COPIES( ' ', 3 ) ||,                            /* 0555 */
         curr.year'-'curr.mon'-'curr.day || COPIES(' ', 2 ) ||,   /* 0556 */
         RIGHT( curr.time, 6, '0' )      || COPIES(' ', 2 ) ||,   /* 0557 */
         RIGHT( EDIT(curr.size), 9 ),,                            /* 0558 */
            GBL.width - 1 ) || dbl.v                              /* 0559 */
                                                                  /* 0560 */
                                                                  /* 0561 */
ln = ln + 1; line.ln =,                                           /* 0562 */
   dbl.bl  || COPIES( dbl.h, GBL.width - 2 ) || dbl.br            /* 0563 */
                                                                  /* 0564 */
line.0 = ln                                                       /* 0565 */
                                                                  /* 0566 */
call LINEOUT GBL.log_file, ' '      /* blank line */              /* 0567 */
do ln = 1 to line.0                                               /* 0568 */
   call WRITE_LOG_FILE_REM_LINE line.ln                           /* 0569 */
end                                                               /* 0570 */
return                                                            /* 0571 */
                                                                  /* 0572 */
                                                                  /* 0573 */
/*------------------------------------------------------------------------*\
|                                                                          |
|               Build & write log file warning header lines                |
|                                                                          |
\*------------------------------------------------------------------------*/
CREATE_LOG_FILE_HEADER:                                           /* 0579 */
   Procedure expose,                                              /* 0580 */
      GBL. (GBL.box_draw)                                         /* 0581 */
                                                                  /* 0582 */
data=''                                                           /* 0583 */
data=data'The following files may not necessarily be duplicates. Their '
data=data'presence only indicates that the files marked for deletion '
data=data'have the same title and an earlier date than than the files '
data=data'they are supposedly replaced by.'                       /* 0587 */
                                                                  /* 0588 */
call LINEOUT GBL.log_file, '@echo off'                            /* 0589 */
                                                                  /* 0590 */
call WRITE_LOG_FILE_REM_LINE,                                     /* 0591 */
   dbl.tl  || COPIES( dbl.h,      GBL.width - 2 ) || dbl.tr       /* 0592 */
call WRITE_LOG_FILE_REM_LINE,                                     /* 0593 */
   dbl.v   || COPIES( ' ',        GBL.width - 2 ) || dbl.v        /* 0594 */
call WRITE_LOG_FILE_REM_LINE,                                     /* 0595 */
   dbl.v   || CENTER( 'WARNING!', GBL.width - 2 ) || dbl.v        /* 0596 */
call WRITE_LOG_FILE_REM_LINE,                                     /* 0597 */
   dbl.v   || COPIES( ' ',        GBL.width - 2 ) || dbl.v        /* 0598 */
call WRITE_LOG_FILE_REM_LINE,                                     /* 0599 */
   dbl.t_r || COPIES( dbl.h,      GBL.width - 2 ) || dbl.t_l      /* 0600 */
call WRITE_LOG_FILE_REM_LINE,                                     /* 0601 */
   dbl.v   || COPIES( ' ',        GBL.width - 2 ) || dbl.v        /* 0602 */
                                                                  /* 0603 */
do while data = ''                                               /* 0604 */
   end_ptr = LASTPOS( ' ', data, GBL.width - 4 ) - 1              /* 0605 */
   if end_ptr = -1 then                                           /* 0606 */
      do                                                          /* 0607 */
         if data = '' then                                       /* 0608 */
            do                                                    /* 0609 */
               output_line =,                                     /* 0610 */
                  LEFT(dbl.v || ' ' || data, GBL.width - 2 ) || ' ' || dbl.v
               call WRITE_LOG_FILE_REM_LINE output_line           /* 0612 */
            end                                                   /* 0613 */
         leave                                                    /* 0614 */
      end                                                         /* 0615 */
   right_space_count = GBL.width - 3 - end_ptr                    /* 0616 */
   output_line =,                                                 /* 0617 */
      dbl.v || ' ' || LEFT( data, end_ptr ) ||,                   /* 0618 */
      COPIES( ' ', right_space_count ) || dbl.v                   /* 0619 */
   call WRITE_LOG_FILE_REM_LINE output_line                       /* 0620 */
                                                                  /* 0621 */
   beg_ptr = end_ptr + 2                                          /* 0622 */
   parse value data with,                                         /* 0623 */
      . =(beg_ptr) data                                           /* 0624 */
end                                                               /* 0625 */
                                                                  /* 0626 */
call WRITE_LOG_FILE_REM_LINE,                                     /* 0627 */
   dbl.v   || COPIES( ' ',        GBL.width - 2 ) || dbl.v        /* 0628 */
call WRITE_LOG_FILE_REM_LINE,                                     /* 0629 */
   dbl.bl  || COPIES( dbl.h,      GBL.width - 2 ) || dbl.br       /* 0630 */
                                                                  /* 0631 */
return                                                            /* 0632 */
                                                                  /* 0633 */
/*----------------------------------------------------------------*\
|                                                                  |
|                        EDIT REXX function                        |
| ---------------------------------------------------------------- |
|                                                                  |
|   The following REXX subroutine / function will return an        |
|   edited integer or decimal value with appropriate               |
|   punctuation inserted appropriately (subject to DIGITS() ).     |
|   The correct punctuation characters are obtained from the       |
|   object system. For example (assuming US notation):             |
|                                                                  |
|         EDIT(12345)                                              |
|                                                                  |
|    will return 12,345; and                                       |
|                                                                  |
|         EDIT(-123456.78)                                         |
|                                                                  |
|    will return -123,456.78                                       |
|                                                                  |
|    The intent behind the creation of this function was speed.    |
|    It is being released to the public domain by the author -     |
|    Dick Goran, author of the "REXX Reference Summary Handbook"   |
|       71154,2002                 CompuServe                      |
|       71154.2002@compuserve.com  Internet                        |
|                                                                  |
|    88/04/24 - originally written for DOS Personal REXX           |
|    93/12/25 - handle negative values & international notation    |
|               (presumes REXXUTIL registered)                     |
|                                                                  |
\*----------------------------------------------------------------*/
EDIT:                                                             /* 0664 */
/* first time here, build translate tables */                     /* 0665 */
SIGNAL OFF NOVALUE                                                /* 0666 */
if LEFT(e1, 1) <> '01'x then                                      /* 0667 */
   do                                                             /* 0668 */
      e1 = XRANGE('01'x, '19'x)                                   /* 0669 */
      e2 = XRANGE('01'x, '03'x) || '19'x ||,                      /* 0670 */
           XRANGE('04'x, '06'x) || '19'x ||,                      /* 0671 */
           XRANGE('07'x, '09'x) || '19'x ||,                      /* 0672 */
           XRANGE('0A'x, '0C'x) || '19'x ||,                      /* 0673 */
           XRANGE('0D'x, '0F'x) || '19'x ||,                      /* 0674 */
           XRANGE('10'x, '12'x) || '19'x ||,                      /* 0675 */
           XRANGE('13'x, '15'x) || '19'x ||,                      /* 0676 */
           XRANGE('16'x, '18'x)                                   /* 0677 */
      /* get punctuation characters from INI file  */             /* 0678 */
      decimal  = STRIP( SysIni( 'USER',,                          /* 0679 */
                                'PM_National',,                   /* 0680 */
                                'sDecimal' ), 'T', '00'x )        /* 0681 */
      thousand = STRIP( SysIni( 'USER',,                          /* 0682 */
                                'PM_National',,                   /* 0683 */
                                'sThousand' ), 'T', '00'x )       /* 0684 */
   end                                                            /* 0685 */
SIGNAL ON NOVALUE                                                 /* 0686 */
                                                                  /* 0687 */
/* return BAD if non-numeric data */                              /* 0688 */
if DATATYPE( ARG(1) ) <> 'NUM' then                               /* 0689 */
   return 'BAD'                                                   /* 0690 */
                                                                  /* 0691 */
/* test and save sign value along with absolute numeric value */  /* 0692 */
if SIGN( ARG(1) ) <> '-1' then                                    /* 0693 */
   sign_character = ''                                            /* 0694 */
else                                                              /* 0695 */
   sign_character = '-'                                           /* 0696 */
absolute_value = ABS( ARG(1) )                                    /* 0697 */
                                                                  /* 0698 */
/* test for and save decimal value indicator */                   /* 0699 */
decimal_position = POS( decimal, absolute_value )                 /* 0700 */
                                                                  /* 0701 */
if decimal_position = 0 then                                      /* 0702 */
   source = RIGHT( absolute_value, LENGTH(e1) - 1 ) || ' '        /* 0703 */
else                                                              /* 0704 */
   source = RIGHT( LEFT( absolute_value, decimal_position - 1 ), LENGTH(e1) - 1 ) || ' '
                                                                  /* 0706 */
if decimal_position = 0 then                                      /* 0707 */
   edited_number =,                                               /* 0708 */
      STRIP( TRANSLATE( TRANSLATE( e2, source, e1), ',', ' '), 'B', ',')
else                                                              /* 0710 */
   edited_number =,                                               /* 0711 */
      STRIP( TRANSLATE( TRANSLATE( e2, source, e1), ',', ' '), 'B', ','),
                      || RIGHT( absolute_value,,                  /* 0713 */
                                LENGTH(absolute_value) - decimal_position + 1)
return sign_character || edited_number                            /* 0715 */
                                                                  /* 0716 */
                                                                  /* 0717 */
!tr! = VALUE('TRACE',,GBL.environment); if !tr! <> '' then do; TRACE(!tr!); nop; end
/*------------------------------------------------------------------------*\
|                                                                          |
|                                End of Job                                |
|                                                                          |
\*------------------------------------------------------------------------*/
EOJ:                                                              /* 0724 */
   Procedure expose,                                              /* 0725 */
      GBL.                                                        /* 0726 */
                                                                  /* 0727 */
if ARG() = 0 then                                                 /* 0728 */
   eoj_rc = 0                                                     /* 0729 */
else                                                              /* 0730 */
   eoj_rc = ARG(1)                                                /* 0731 */
                                                                  /* 0732 */
elapsed_time = TIME('E')            /* get elapsed time - sssss.uuuuu */
parse value elapsed_time with seconds '.' micro_seconds           /* 0734 */
if LEFT( micro_seconds, 1, 1 ) >= 5 then                          /* 0735 */
   seconds = seconds + 1                                          /* 0736 */
ss = FORMAT( seconds // 60, 2 )                                   /* 0737 */
minutes = ( seconds - ss ) / 60                                   /* 0738 */
mm = FORMAT( minutes // 60, 2 )                                   /* 0739 */
hh = FORMAT( ( minutes - mm ) / 60, 2 )                           /* 0740 */
duration = hh':'mm':'ss                                           /* 0741 */
                                                                  /* 0742 */
program_name = TRANSLATE( FILESPEC( 'N', GBL.program_path_and_name ) )
say 'EOJ  ' program_name 'at' TIME('N') ||,                       /* 0744 */
    ', duration' TRANSLATE( duration, '0', ' ' )                  /* 0745 */
exit eoj_rc                                                       /* 0746 */
                                                                  /* 0747 */
/*------------------------------------------------------------------------*\
|                                                                          |
|          Write line (passed as ARG) to log file preceded by REM          |
|                                                                          |
\*------------------------------------------------------------------------*/
WRITE_LOG_FILE_REM_LINE:                                          /* 0753 */
   Procedure expose,                                              /* 0754 */
      GBL.                                                        /* 0755 */
                                                                  /* 0756 */
call LINEOUT GBL.log_file, 'rem ' || ARG(1)                       /* 0757 */
return                                                            /* 0758 */
                                                                  /* 0759 */
                                                                  /* 0760 */
/*------------------------------------------------------------------------*\
|                                                                          |
|                   Register external function routines                    |
|                                                                          |
\*------------------------------------------------------------------------*/
REGISTER_REQUIRED_FUNCTIONS:                                      /* 0766 */
   Procedure expose GBL.REXX_version                              /* 0767 */
                                                                  /* 0768 */
/*----------------------------------------*\                      /* 0769 */
|  Load REXXUtil External Function Module  |                      /* 0770 */
\*----------------------------------------*/                      /* 0771 */
module             = 'REXXUTIL'                                   /* 0772 */
entry_name         = 'SysLoadFuncs'                               /* 0773 */
function_name      = 'SysLoadFuncs'                               /* 0774 */
anticipated_return = ''                                           /* 0775 */
call REGISTER_ROUTINE function_name, module, entry_name, anticipated_return
                                                                  /* 0777 */
/*-----------------------------------*\                           /* 0778 */
|  Load the REXXLIB Function Package  |                           /* 0779 */
\*-----------------------------------*/                           /* 0780 */
if GBL.REXX_version = 'REXX/Personal' then                        /* 0781 */
   do                                                             /* 0782 */
      module = 'qrexxlib'                                         /* 0783 */
   end                                                            /* 0784 */
else                                                              /* 0785 */
   do                                                             /* 0786 */
      module = 'rexxlib'                                          /* 0787 */
   end                                                            /* 0788 */
entry_name         = 'rexxlibregister'                            /* 0789 */
function_name      = 'RexxLibRegister'                            /* 0790 */
anticipated_return = '1'                                          /* 0791 */
call REGISTER_ROUTINE function_name, module, entry_name, anticipated_return
                                                                  /* 0793 */
return                                                            /* 0794 */
                                                                  /* 0795 */
                                                                  /* 0796 */
/*---------------------*\                                         /* 0797 */
|  Register Subroutine  |                                         /* 0798 */
\*---------------------*/                                         /* 0799 */
REGISTER_ROUTINE:                                                 /* 0800 */
   Procedure                                                      /* 0801 */
                                                                  /* 0802 */
parse ARG  function_name,,                                        /* 0803 */
           module,,                                               /* 0804 */
           entry_name,,                                           /* 0805 */
           anticipated_return                                     /* 0806 */
                                                                  /* 0807 */
if RxFuncQuery(function_name) = 0 then return      /* function registered */
                                                                  /* 0809 */
if LENGTH(module) > 8 then                                        /* 0810 */
   do                                                             /* 0811 */
      dll_drive = FILESPEC( 'D', module )                         /* 0812 */
      dll_path  = STRIP( FILESPEC( 'P', module ), 'T', '\' )      /* 0813 */
      module    = FILESPEC( 'N', module )                         /* 0814 */
      '@' || dll_drive                                            /* 0815 */
      '@cd' dll_drive || dll_path                                 /* 0816 */
   end                                                            /* 0817 */
else                                                              /* 0818 */
   do                                                             /* 0819 */
      dll_drive = ''                                              /* 0820 */
   end                                                            /* 0821 */
                                                                  /* 0822 */
parse var module module_fname '.' module_fext                     /* 0823 */
if RxFuncAdd( function_name, module_fname, entry_name ) = 0 then  /* 0824 */
   do                                                             /* 0825 */
      register_call = 'call' function_name                        /* 0826 */
      interpret register_call                                     /* 0827 */
      if WORD( RESULT, 1 ) <> WORD( anticipated_return, 1 ) then  /* 0828 */
         do                                                       /* 0829 */
            Say function_name 'returned' RESULT '-',              /* 0830 */
                                         anticipated_return 'was expected'
            exit 255                                              /* 0832 */
         end                                                      /* 0833 */
   end                                                            /* 0834 */
else                                                              /* 0835 */
   do                                                             /* 0836 */
      Say 'RxFuncAdd returned' RESULT 'registering' module        /* 0837 */
      exit 254                                                    /* 0838 */
   end                                                            /* 0839 */
if dll_drive <> '' then                                           /* 0840 */
   do                                                             /* 0841 */
      Parse Source . . GBL.program_path_and_name                  /* 0842 */
      '@' || LEFT( GBL.program_path_and_name, 2 )                 /* 0843 */
   end                                                            /* 0844 */
return                                                            /* 0845 */
                                                                  /* 0846 */
/*------------------------------------------------------------------------*\
|                                                                          |
|                              Trap Routines                               |
|                                                                          |
\*------------------------------------------------------------------------*/
ERROR:   call TRAP_PROCESSING SIGL, 'ERROR',   RC                 /* 0852 */
FAILURE: call TRAP_PROCESSING SIGL, 'FAILURE', RC                 /* 0853 */
HALT:    call TRAP_PROCESSING SIGL, 'HALT',    ''                 /* 0854 */
NOVALUE: call TRAP_PROCESSING SIGL, 'NOVALUE', ''                 /* 0855 */
SYNTAX:  call TRAP_PROCESSING SIGL, 'SYNTAX',  RC                 /* 0856 */
                                                                  /* 0857 */
/* Rev. 95/06/27 */                                               /* 0858 */
TRAP_PROCESSING:                                                  /* 0859 */
   parse Source . . TRAP.path_and_program                         /* 0860 */
   trap.line_nbr = ARG(1)                                         /* 0861 */
   if POS( ':', TRAP.path_and_program ) > 0 then                  /* 0862 */
      /* get source line if it is available */                    /* 0863 */
      do t = 1                                                    /* 0864 */
         trap_source_line.t =  SOURCELINE( trap.line_nbr )        /* 0865 */
         trap_source_line.0 = t                                   /* 0866 */
         trap.line_nbr      = trap.line_nbr + 1                   /* 0867 */
         if RIGHT( SOURCELINE( trap.line_nbr ), 1 ) = ',' then   /* 0868 */
            do                                                    /* 0869 */
               if t > 1 then                                      /* 0870 */
                  do                                              /* 0871 */
                     t = t + 1                                    /* 0872 */
                     trap_source_line.t =  SOURCELINE( trap.line_nbr )
                     trap_source_line.0 = t                       /* 0874 */
                  end                                             /* 0875 */
               leave                                              /* 0876 */
            end                                                   /* 0877 */
      end                                                         /* 0878 */
   else                                                           /* 0879 */
      /* program is running in macrospace */                      /* 0880 */
      do                                                          /* 0881 */
         TRAP.path_and_program = VALUE( 'TEMP',, 'OS2ENVIRONMENT' ) ||,
                                 '\' || TRAP.path_and_program     /* 0883 */
         trap_source_line.1 = 'Source line is not available.'     /* 0884 */
         trap_source_line.0 = 1                                   /* 0885 */
      end                                                         /* 0886 */
                                                                  /* 0887 */
   parse value FILESPEC( 'N', TRAP.path_and_program ) with,       /* 0888 */
      TRAP.fn '.' TRAP.fe                                         /* 0889 */
   trap_file_name = FILESPEC( 'D', TRAP.path_and_program ) ||,    /* 0890 */
                    FILESPEC( 'P', TRAP.path_and_program ) ||,    /* 0891 */
                    TRAP.fn || '.' || 'DMP'                       /* 0892 */
                                                                  /* 0893 */
   /*------------------------------------------*\                 /* 0894 */
   |  check for reason not to create .DMP file  |                 /* 0895 */
   \*------------------------------------------*/                 /* 0896 */
   if ARG(2) = 'HALT' then                                        /* 0897 */
      do                                                          /* 0898 */
         trap_file_name = ''                                      /* 0899 */
      end                                                         /* 0900 */
   if RxFuncQuery( 'VARDUMP' ) <> 0 then                          /* 0901 */
      do                                                          /* 0902 */
         trap_file_name = ''                                      /* 0903 */
      end                                                         /* 0904 */
   if POS( ':', trap_file_name ) = 0 then                         /* 0905 */
      do                                                          /* 0906 */
         trap_file_name = ''                                      /* 0907 */
      end                                                         /* 0908 */
                                                                  /* 0909 */
   /*------------------------*\                                   /* 0910 */
   |  Build trap message box  |                                   /* 0911 */
   \*------------------------*/                                   /* 0912 */
   dbl.h    = 'CD'x                 /*  double line - horizontal   */
   dbl.v    = 'BA'x                 /*  double line - vertical     */
   dbl.bl   = 'C8'x                 /*  double line - bottom left  */
   dbl.br   = 'BC'x                 /*  double line - bottom right */
   dbl.tl   = 'C9'x                 /*  double line - top left     */
   dbl.tr   = 'BB'x                 /*  double line - top right    */
   trap.red = '1B'x || '[1;37;41m'  /* bright white on red          */
   trap.dul = '1B'x || '[0m'        /* reset to normal              */
                                                                  /* 0921 */
   say ' '                                                        /* 0922 */
   trap_error_description =,                                      /* 0923 */
      'Error line = ' || ARG(1) ||,                               /* 0924 */
      '; ' ||,                                                    /* 0925 */
      ARG(2) ||,                                                  /* 0926 */
      ' error.'                                                   /* 0927 */
   if ARG(3) <> '' then                                           /* 0928 */
      trap_error_description = trap_error_description ||,         /* 0929 */
                               '  Return code = ' || ARG(3)       /* 0930 */
   trap.width = MAX( 74, LENGTH( trap_error_description ) )       /* 0931 */
   say trap.red || dbl.tl || COPIES( dbl.h,trap.width + 2 ) || dbl.tr
   say trap.red || dbl.v  || COPIES( ' ',  trap.width + 2 ) || dbl.v
   say trap.red || dbl.v CENTER( TRAP.fn'.CMD',trap.width )    dbl.v
   say trap.red || dbl.v CENTER( trap_error_description, trap.width ) dbl.v
   if trap_file_name <> '' then                                   /* 0936 */
      do                                                          /* 0937 */
   say trap.red || dbl.v  || COPIES( ' ',  trap.width + 2 ) || dbl.v
   say trap.red || dbl.v     CENTER( 'See: ' || trap_file_name,,  /* 0939 */
                                     trap.width )  dbl.v          /* 0940 */
      end                                                         /* 0941 */
   say trap.red || dbl.v  || COPIES( ' ',  trap.width + 2 ) || dbl.v
   say trap.red || dbl.bl || COPIES( dbl.h,trap.width + 2 ) || dbl.br
   say trap.red || COPIES( ' ', trap.width + 4 )                  /* 0944 */
   say trap.red || LEFT( 'Source line at time of trap:', trap.width + 4 )
   do t = 1 to trap_source_line.0                                 /* 0946 */
      say trap.red || LEFT( '   ' || trap_source_line.t, trap.width + 4 )
   end                                                            /* 0948 */
   say trap.red || COPIES( ' ', trap.width + 4 )                  /* 0949 */
                                                                  /* 0950 */
   /*---------------------------------*\                          /* 0951 */
   |  Create .DMP file if appropriate  |                          /* 0952 */
   \*---------------------------------*/                          /* 0953 */
   if trap_file_name <> '' then                                   /* 0954 */
      do                                                          /* 0955 */
         call SysFileDelete trap_file_name                        /* 0956 */
         /* remove meaningless labels from dump for clarity */    /* 0957 */
         drop dbl. TRAP. RC RESULT SIGL !tr!                      /* 0958 */
         call VARDUMP trap_file_name  /* write variables to program.DMP file */
      end                                                         /* 0960 */
   exit 253                                                       /* 0961 */
/*---------  REXX Cross Reference  - Created: 04/07/95 1:00am ----------*\

---- VARIABLES ----
!tr!                0718<  0718   0718   0958
GBL.                0052<  0294   0501   0518   0581   0726   0755
GBL.REXX_version    0054   0767   0781
GBL.TREE_STEM.0     0168<  0171   0224   0414   0415<  0434   0448
GBL.TREE_STEM.ts    0225   0231   0416<  0423   0451   0465<
GBL.bksp            0136<  0256   0264   0427   0489   0503
GBL.boot_drive      0059<
GBL.box_draw        0084<  0294   0518   0581
GBL.calling_environment
                    0056
GBL.command_line    0053   0156
GBL.environment     0058<  0059   0718
GBL.iDate_character
                    0147<  0342   0351   0360
GBL.id              0079<  0177   0247
GBL.idate_character
                    0372<
GBL.log_file        0271   0272   0441   0473   0567   0589   0757
GBL.log_file_line_count
                    0076<  0269   0468   0474<  0474
GBL.operating_system
                    0055
GBL.program_fe      0067
GBL.program_fn      0066   0074   0155
GBL.program_name    0061<  0065   0069
GBL.program_path    0062<  0073   0154
GBL.program_path_and_name
                    0057   0061   0062   0063   0743   0842   0843
GBL.program_version
                    0060<
GBL.progress_list   0137<  0504   0506
GBL.progress_subscript
                    0138<  0504   0505<  0505   0506   0507<
GBL.sDate_character
                    0143<  0145<  0346   0348   0355   0357   0364   0366
GBL.sort_max_length
                    0077<  0423<  0423   0436
GBL.title_max_length
                    0078<  0409<  0409
GBL.tree_file       0158   0161   0164   0488
GBL.tree_stem.0     0444
GBL.width           0085<  0532   0534   0536   0540   0546   0549   0553
                    0559   0563   0592   0594   0596   0598   0600   0602
                    0605   0611   0616   0628   0630
GBl.log_file        0072<
GBl.tree_file       0153<
RC                  0852   0853   0856   0958
RESULT              0200   0254   0828   0830   0837   0958
SIGL                0852   0853   0854   0855   0856   0958
TRAP.               0958
TRAP.fe             0889
TRAP.fn             0889   0892   0934
TRAP.path_and_program
                    0860   0862   0882<  0883   0888   0890   0891
Version             0054
absolute_value      0697<  0700   0703   0705   0713   0714
anticipated_return  0775<  0776   0791<  0792   0806   0828   0831
b                   0279
beg_ptr             0622<  0624
box_line.0          0278<  0279
box_line.1          0275<
box_line.2          0274<  0277
box_line.3          0272<  0274   0275   0276
box_line.4          0277<
box_line.5          0276<
box_line.b          0280
curr.               0519
curr.crc            0456   0466   0480
curr.date           0453   0481   0521
curr.day            0524   0556
curr.mon            0523   0556
curr.path_and_file_name
                    0457   0473   0484   0553
curr.size           0455   0483   0558
curr.time           0454   0482   0557
curr.title          0452   0458<  0458   0460   0479   0534
curr.year           0522   0556
data                0583<  0584<  0584   0585<  0585   0586<  0586   0587<
                    0587   0604   0605   0608   0611   0618   0623   0624
dbl.                0958
dbl.bl              0089<  0276   0563   0630   0915<  0943
dbl.br              0090<  0276   0563   0630   0916<  0943
dbl.cross           0097<
dbl.h               0087<  0275   0276   0532   0563   0592   0600   0630
                    0913<  0932   0943
dbl.t_d             0093<
dbl.t_l             0094<  0600
dbl.t_r             0095<  0600
dbl.t_u             0096<
dbl.tl              0091<  0275   0532   0592   0917<  0932
dbl.tr              0092<  0275   0532   0592   0918<  0932
dbl.v               0088<  0272   0273   0274   0274   0534   0534   0539
                    0540   0542   0546   0549   0549   0552   0553   0555
                    0559   0594   0594   0596   0596   0598   0598   0602
                    0602   0611   0611   0618   0619   0628   0628   0914<
                    0933   0933   0934   0934   0935   0935   0938   0938
                    0939   0940   0942   0942
decimal             0679<  0700
decimal_position    0700<  0702   0705   0707   0714
dll_drive           0812<  0815   0816   0820<  0840
dll_path            0813<  0816
drive_letter_colon  0300   0306   0308   0309   0318   0321
drive_list          0296<  0298   0299   0301
duration            0741<  0745
e1                  0667   0669<  0703   0705   0709   0712
e2                  0670<  0709   0712
edited_number       0708<  0711<  0715
elapsed_time        0733<  0734
end_ptr             0605<  0606   0616   0618   0622
entry_name          0773<  0776   0789<  0792   0805   0824
eoj_rc              0729<  0731<  0746
folder_class        0179<  0195
folder_id           0177<  0188   0192   0251
folder_location     0178<  0197
folder_setup        0181<  0198
folder_title        0180<  0182   0196   0203
function_name       0774<  0776   0790<  0792   0803   0808   0824   0826
                    0830
hh                  0740<  0741
inf_class           0214<  0249
inf_crc             0236   0247
inf_date            0233
inf_path_and_file_name
                    0237   0242   0245   0258
inf_setup           0215<  0244
inf_size            0235
inf_time            0234
inf_title           0232   0240
line.0              0565<  0568
line.ln             0531<  0533<  0535<  0538<  0541<  0548<  0551<  0554<
                    0562<  0569
ln                  0530<  0531<  0531   0533<  0533   0535<  0535   0538<
                    0538   0541<  0541   0548<  0548   0551<  0551   0554<
                    0554   0562<  0562   0565   0568
lowercase           0211<  0242
micro_seconds       0734   0735
minutes             0738<  0739   0740
mm                  0739<  0740   0741
module              0772<  0776   0783<  0787<  0792   0804   0810   0812
                    0813   0814<  0814   0823   0837
module_fext         0823
module_fname        0823   0824
output_line         0610<  0612   0617<  0620
prev.               0442<  0519
prev.crc            0466   0480<
prev.date           0481<  0525
prev.day            0528   0543
prev.mon            0527   0543
prev.path_and_file_name
                    0484<  0540
prev.size           0483<  0545
prev.time           0482<  0544
prev.title          0460   0479<
prev.year           0526   0543
program_name        0743<  0744
register_call       0826<  0827
remove_characters   0404<  0406   0407
right_space_count   0616<  0619
seconds             0734   0736<  0736   0737   0738
sgh.bl              0113<
sgh.br              0114<
sgh.cross           0121<
sgh.h               0111<
sgh.t_d             0117<
sgh.t_l             0118<  0536
sgh.t_r             0119<  0536
sgh.t_u             0120<
sgh.tl              0115<
sgh.tr              0116<
sgh.v               0112<
sgl.bl              0101<
sgl.br              0102<
sgl.cross           0109<
sgl.h               0099<  0536
sgl.t_d             0105<
sgl.t_l             0106<
sgl.t_r             0107<
sgl.t_u             0108<
sgl.tl              0103<
sgl.tr              0104<
sgl.v               0100<
sgv.bl              0125<
sgv.br              0126<
sgv.cross           0133<
sgv.h               0123<
sgv.t_d             0129<
sgv.t_l             0130<
sgv.t_r             0131<
sgv.t_u             0132<
sgv.tl              0127<
sgv.tr              0128<
sgv.v               0124<
sign_character      0694<  0696<  0715
ss                  0737<  0738   0741
t                   0864   0866   0870   0872<  0872   0874   0946
temp                0330   0390   0425
temp_area           0385<  0387   0392
temp_attr           0337
temp_date           0334   0344   0353   0362
temp_dd             0347   0354   0367   0418
temp_mm             0345   0356   0365   0418
temp_path_and_file_name
                    0338   0339<  0339   0385   0386   0398   0421   0422
temp_setup          0244<  0252
temp_size           0336   0420
temp_stem.0         0322   0328   0330
temp_stem.temp      0333
temp_time           0335   0419
temp_title          0394   0396   0398<  0405<  0405   0408<  0408   0409
                    0417
temp_yy             0349   0358   0363   0376   0377   0379
temp_yyyy           0376<  0379<  0418
thousand            0682<
title_line_1        0239<  0246   0250
title_line_2        0241<  0246   0250
trap.dul            0920<
trap.line_nbr       0861<  0865   0867<  0867   0868   0873
trap.red            0919<  0932   0933   0934   0935   0938   0939   0942
                    0943   0944   0945   0947   0949
trap.width          0931<  0932   0933   0934   0935   0938   0940   0942
                    0943   0944   0945   0947   0949
trap_error_description
                    0923<  0929<  0929   0931   0935
trap_file_name      0890<  0899<  0903<  0905   0907<  0936   0939   0954
                    0956   0959
trap_source_line.0  0866<  0874<  0885<  0946
trap_source_line.1  0884<
trap_source_line.t  0865<  0873<  0947
ts                  0224   0227   0414<  0415   0448   0476
uppercase           0212<  0242
with                0231   0299   0333   0344   0353   0362   0392   0451
                    0521   0525   0623   0734   0888
x                   0136   0144   0148   0225   0256   0264   0324   0387
                    0395   0427   0465   0489   0667   0669   0669   0670
                    0670   0670   0671   0671   0671   0672   0672   0672
                    0673   0673   0673   0674   0674   0674   0675   0675
                    0675   0676   0676   0676   0677   0677   0681   0684
                    0913   0914   0915   0916   0917   0918   0919   0920

---- LABELS ----
ADVANCE_PROGRESS_INDICATOR
                    0230   0332   0449   0499:
BUILD_TREE_STEM     0169   0292:
CREATE_LOG_FILE_DETAIL
                    0472   0516:
CREATE_LOG_FILE_HEADER
                    0470   0579:
EDIT                0664:
EOJ                 0205   0284   0373   0724:
ERROR               0046   0852:
FAILURE             0047   0853:
HALT                0048   0854:
NOVALUE             0049   0666   0686   0855:
REGISTER_REQUIRED_FUNCTIONS
                    0070   0766:
REGISTER_ROUTINE    0776   0792   0800:
SYNTAX              0050   0856:
TRAP_PROCESSING     0852   0853   0854   0855   0856   0859:
WRITE_LOG_FILE_REM_LINE
                    0569   0591   0593   0595   0597   0599   0601   0612
                    0620   0627   0629   0753:

---- FUNCTIONS ----
ABBREV              0156
ABS                 0697
ARG                 0689   0693   0697   0728   0731   0757   0861   0897
                    0924   0926   0928   0930
ARRAYSORT           0171   0434
CENTER              0534   0549   0596   0934   0935   0939
CHARIN              0385
CHAROUT             0191   0256   0264   0316   0324   0328   0427   0443
                    0489   0503
COPIES              0192   0274   0275   0276   0280   0406   0532   0536
                    0542   0543   0544   0555   0556   0557   0563   0592
                    0594   0598   0600   0602   0619   0628   0630   0932
                    0933   0938   0942   0943   0944   0949
CVREAD              0164
CVWRITE             0488
DATATYPE            0689
DOSDISK             0306
DOSFILESYS          0308   0309
EDIT                0545   0558
FILECRC             0421
FILESPEC            0061   0062   0063   0398   0743   0812   0813   0814
                    0888   0890   0891
FORMAT              0737   0739   0740
LASTPOS             0605
LEFT                0059   0387   0417   0539   0542   0552   0555   0611
                    0618   0667   0705   0735   0843   0945   0947
LENGTH              0274   0275   0276   0406   0409   0423   0506   0703
                    0705   0714   0810   0931
LINEOUT             0473   0567   0589   0757
MAX                 0409   0423   0931
POS                 0700   0862   0905
RIGHT               0328   0418   0418   0419   0544   0545   0557   0558
                    0703   0705   0713   0868
RxFuncAdd           0824
RxFuncQuery         0808   0901
SIGN                0693
SOURCELINE          0865   0868   0873
SPACE               0408
STREAM              0161   0271   0386
STRIP               0144   0148   0240   0339   0458   0679   0682   0709
                    0712   0813
SUBSTR              0504
SysCreateObject     0195   0249
SysCurState         0193   0265   0329   0428   0446   0490
SysDriveMap         0296
SysFileDelete       0158   0441   0956
SysFileTree         0321
SysIni              0144   0148   0679   0682
TIME                0068   0069   0733   0744
TRACE               0718
TRANSLATE           0069   0074   0155   0156   0242   0398   0405   0709
                    0709   0712   0712   0743   0745
VALUE               0059   0718   0882
VARDUMP             0959
WORD                0828   0828
XRANGE              0669   0670   0671   0672   0673   0674   0675   0676
                    0677

\*-------------------  End of REXX Cross Reference  -------------------*/
